"use client";

import { IoRefresh } from "react-icons/io5";
import { SearchIcon } from "@chakra-ui/icons";
import {
  Box,
  Button,
  Flex,
  FormControl,
  FormLabel,
  Icon,
  Input,
  InputGroup,
  InputLeftElement,
  Text,
  useColorModeValue,
} from "@chakra-ui/react";
import React, { useMemo, useState, useDeferredValue } from "react";
import Loading from "@/app/loading";
import Card from "./card";
import Pagination from "../../components/pagination";
import MultiSelect from "@/components/fields/MultiSelect";
import { SelectWrapper } from "@/components/fields/Fields";
import { Assignment } from "@/types/assignment";


interface CardListProps {
  data: Assignment[];
  draftDetails: any[];
  isLoading: boolean;
  handleSync: () => void;
}

interface MultiSelectOption {
  value: number | string;
  label: number | string;
}

const CardList = ({
  data = [],
  draftDetails = [],
  isLoading = false,
  handleSync,
}: CardListProps) => {
  const [searchQuery, setSearchQuery] = useState("");
  const [currentPage, setCurrentPage] = useState(1);
  const [sortOrder, setSortOrder] = useState<"asc" | "desc">("asc");
  const [selectedCycleDays, setSelectedCycleDays] = useState<
    MultiSelectOption[]
  >([]);
  const itemsPerPage = 20;
  const searchDeferredQuery = useDeferredValue(searchQuery);

  const handleSearchChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setSearchQuery(e.target.value);
  };

  const handleSortChange = (value: string | any[]) => {
    setSortOrder(value as "asc" | "desc");
  };

  const handleCycleDayChange = (selectedOptions: MultiSelectOption[]) => {
    setSelectedCycleDays(selectedOptions);
  };

  const filteredData = useMemo(() => {
    let result = data;

    if (searchDeferredQuery) {
      const lowerCaseQuery = searchDeferredQuery.toLowerCase();
      result = result?.filter(
        (consumer) =>
          consumer?.case?.name?.toLowerCase()?.includes(lowerCaseQuery) ||
          consumer?.case?.contract?.toLowerCase()?.includes(lowerCaseQuery)
      );
    }
    if (draftDetails?.length > 0) {
      result = result?.filter((consumer) => {
        return !draftDetails?.some((draft) => {
          return draft?.assignment_id == consumer?.id;
        });
      });
    }

    if (selectedCycleDays?.length > 0) {
      const selectedCycleDayValues = selectedCycleDays?.map(
        (option) => option.value
      );
      result = result?.filter((consumer) =>
        selectedCycleDayValues?.includes(consumer?.case?.cycle_day?.cycle_day)
      );
    }

    return result;
  }, [searchDeferredQuery, selectedCycleDays, draftDetails, data]);

  const sortedData = useMemo(() => {
    if (!Array.isArray(filteredData)) return []; // Return an empty array if filteredData is not an array
    return filteredData.sort((a, b) => {
      const mruA = a?.case?.mru;
      const mruB = b?.case?.mru;

      if (sortOrder === "asc") {
        return mruA > mruB ? 1 : -1;
      } else {
        return mruA < mruB ? 1 : -1;
      }
    });
  }, [filteredData, sortOrder]);

  // Calculate data for the current page
  const startIndex = (currentPage - 1) * itemsPerPage;
  const endIndex = startIndex + itemsPerPage;
  const currentData = sortedData.slice(startIndex, endIndex);

  // Handle page changes
  const handlePageChange = (newPage: number) => {
    setCurrentPage(newPage);
  };

  const totalPages = Math.ceil(filteredData.length / itemsPerPage);

  // Get distinct cycle days for the dropdown options
  const cycleDays: MultiSelectOption[] = useMemo(() => {
    if (!Array.isArray(data)) return []; // Return an empty array if data is not an array

    const cycleDaySet = new Set(
      data
        .map((consumer) => consumer?.case?.cycle_day?.cycle_day)
        .filter((cycle_day) => cycle_day !== null && cycle_day !== undefined) // Exclude null/undefined
    );
    
    return Array.from(cycleDaySet).map((cycle_day) => ({
      label: cycle_day, 
      value: cycle_day,
    }));
  }, [data]);

  return (
    <>
      <Flex alignItems="center" justifyContent="space-between">
        <Button onClick={handleSync}>
          <Icon as={IoRefresh} width="20px" height="20px" color="inherit" />
          &nbsp; Sync
        </Button>
        <InputGroup width="300px">
          <InputLeftElement pointerEvents="none">
            <SearchIcon color={useColorModeValue("black", "white")} />
          </InputLeftElement>
          <Input
            placeholder="Search..."
            variant="outline"
            focusBorderColor="blue.500"
            size="md"
            type="search"
            color={useColorModeValue("black", "white")}
            onChange={handleSearchChange}
          />
        </InputGroup>
      </Flex>
      <Flex align={"center"} justify={"space-between"} gap={3} mt={4}>
        <FormControl w="50%">
          <FormLabel fontSize="sm" fontWeight="500" mb="8px">
            Sort by MRU &nbsp;
          </FormLabel>
          <SelectWrapper
            form={false}
            fieldName="remarks"
            id="remarks"
            variant="auth"
            w="full"
            mb="2px"
            defaultValue={""}
            fontSize="sm"
            type="text"
            fontWeight="500"
            updateOnParent={handleSortChange}
            options={[
              { label: "Asc", value: "asc" },
              { label: "Desc", value: "desc" },
            ]}
          />
        </FormControl>
        <Box width="50%">
          <MultiSelect
            id="cycle-day-select"
            fieldName="cycleDay"
            label="Cycle Day"
            defaultValue={selectedCycleDays}
            options={cycleDays?.length > 0 ? cycleDays : []}
            placeholder="Select Cycle Days"
            onChange={handleCycleDayChange}
          />
        </Box>
      </Flex>

      <br />

      {isLoading ? (
        <Loading />
      ) : currentData?.length > 0 ? (
        currentData?.map((consumer, index) => (
          <Card key={index} consumer={consumer} />
        ))
      ) : (
        <Box mt="70px" display="flex" justifyContent="center">
          <Text>No Data Found</Text>
        </Box>
      )}

      {!isLoading && totalPages > 1 && (
        <Pagination
          currentPage={currentPage}
          totalPages={totalPages}
          onPageChange={handlePageChange}
        />
      )}
    </>
  );
};

export default React.memo(CardList);
